/* SPDX-License-Identifier: GPL-2.0+ */

#ifndef __DT_BINDINGS_CLOCK_BCM6362_H
#define __DT_BINDINGS_CLOCK_BCM6362_H

#define BCM6362_CLK_ADSL_QPROC	1
#define BCM6362_CLK_ADSL_AFE	2
#define BCM6362_CLK_ADSL	3
#define BCM6362_CLK_MIPS	4
#define BCM6362_CLK_WLAN_OCP	5
#define BCM6362_CLK_SWPKT_USB	7
#define BCM6362_CLK_SWPKT_SAR	8
#define BCM6362_CLK_SAR		9
#define BCM6362_CLK_ROBOSW	10
#define BCM6362_CLK_PCM		11
#define BCM6362_CLK_USBD	12
#define BCM6362_CLK_USBH	13
#define BCM6362_CLK_IPSEC	14
#define BCM6362_CLK_SPI		15
#define BCM6362_CLK_HSSPI	16
#define BCM6362_CLK_PCIE	17
#define BCM6362_CLK_FAP		18
#define BCM6362_CLK_PHYMIPS	19
#define BCM6362_CLK_NAND	20

#endif /* __DT_BINDINGS_CLOCK_BCM6362_H */
