savedcmd_scripts/dtc/fdtoverlay := gcc   -o scripts/dtc/fdtoverlay scripts/dtc/libfdt/fdt.o scripts/dtc/libfdt/fdt_ro.o scripts/dtc/libfdt/fdt_wip.o scripts/dtc/libfdt/fdt_sw.o scripts/dtc/libfdt/fdt_rw.o scripts/dtc/libfdt/fdt_strerror.o scripts/dtc/libfdt/fdt_empty_tree.o scripts/dtc/libfdt/fdt_addresses.o scripts/dtc/libfdt/fdt_overlay.o scripts/dtc/fdtoverlay.o scripts/dtc/util.o   
