/*
 * Copyright (c) 2014-2016, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 */
#ifndef _LINUX_NDCTL_H
#define _LINUX_NDCTL_H

#include <uapi/linux/ndctl.h>

enum {
	ND_MIN_NAMESPACE_SIZE = PAGE_SIZE,
};

#endif /* _LINUX_NDCTL_H */
